@@ -18,9 +18,11 @@ except ImportError:
 import dask.array as da
 
 # local modules
-from xrspatial.utils import has_cuda
 from xrspatial.utils import cuda_args
 from xrspatial.utils import get_dataarray_resolution
+from xrspatial.utils import ArrayTypeFunctionMapping
+from xrspatial.utils import not_implemented_func
+
 from .perlin import _perlin, _perlin_gpu
 
 
@@ -216,34 +218,21 @@ def generate_terrain(agg: xr.DataArray,
     .. plot::
        :include-source:
 
-        import numpy as np
-        import xarray as xr
-        from xrspatial import generate_terrain
+        >>> import numpy as np
+        >>> import xarray as xr
+        >>> from xrspatial import generate_terrain
 
-        W = 4
-        H = 3
-        data = np.zeros((H, W), dtype=np.float32)
-        raster = xr.DataArray(data, dims=['y', 'x'])
+        >>> W = 400
+        >>> H = 300
+        >>> data = np.zeros((H, W), dtype=np.float32)
+        >>> raster = xr.DataArray(data, dims=['y', 'x'])
+        >>> xrange = (-20e6, 20e6)
+        >>> yrange = (-20e6, 20e6)
+        >>> seed = 2
+        >>> zfactor = 10
 
-        xrange = (-20e6, 20e6)
-        yrange = (-20e6, 20e6)
-        seed = 2
-        zfactor = 10
-
-        terrain = generate_terrain(raster, xrange, yrange, seed, zfactor)
-
-    .. sourcecode:: python
-
-        >>> print(terrain)
-        <xarray.DataArray 'terrain' (y: 3, x: 4)>
-        array([[ 6.8067746,  5.263137 ,  4.664292 ,  6.821344 ],
-               [ 7.4834156,  6.9849734,  4.3545456,  0.       ],
-               [ 6.7674546, 10.       ,  7.0946655,  7.015267 ]], dtype=float32)  # noqa
-        Coordinates:
-          * x        (x) float64 -1.5e+07 -5e+06 5e+06 1.5e+07
-          * y        (y) float64 -1.333e+07 0.0 1.333e+07
-        Attributes:
-            res:      (10000000.0, -13333333.333333334)
+        >>> terrain = generate_terrain(raster, xrange, yrange, seed, zfactor)
+        >>> terrain.plot.imshow()
     """
 
     height, width = agg.shape
@@ -264,24 +253,15 @@ def generate_terrain(agg: xr.DataArray,
     y_range_scaled = (_scale(y_range[0], full_yrange, (0.0, 1.0)),
                       _scale(y_range[1], full_yrange, (0.0, 1.0)))
 
-    # numpy case
-    if isinstance(agg.data, np.ndarray):
-        out = _terrain_numpy(
-            agg.data, seed, x_range_scaled, y_range_scaled, zfactor
+    mapper = ArrayTypeFunctionMapping(
+        numpy_func=_terrain_numpy,
+        cupy_func=_terrain_cupy,
+        dask_func=_terrain_dask_numpy,
+        dask_cupy_func=lambda *args: not_implemented_func(
+            *args, messages='generate_terrain() does not support dask with cupy backed DataArray'  # noqa
         )
-    # cupy case
-    elif has_cuda() and isinstance(agg.data, cupy.ndarray):
-        out = _terrain_cupy(
-            agg.data, seed, x_range_scaled, y_range_scaled, zfactor
-        )
-    # dask + numpy case
-    elif isinstance(agg.data, da.Array):
-        out = _terrain_dask_numpy(
-            agg.data, seed, x_range_scaled, y_range_scaled, zfactor
-        )
-    else:
-        raise TypeError('Unsupported Array Type: {}'.format(type(agg.data)))
-
+    )
+    out = mapper(agg)(agg.data, seed, x_range_scaled, y_range_scaled, zfactor)
     canvas = ds.Canvas(
         plot_width=width, plot_height=height, x_range=x_range, y_range=y_range
     )
